#! /usr/bin/env python

# Sketch - A Python-based interactive drawing program
# Copyright (C) 2002 by Bernhard Herzog
# 
# Parts of this utility are Copyright (C) 2003 by David Boddie
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Usage: impconvert <Input file> <Output directory> <File extension>
                     [-s <Starting page>] [-f <Finishing page>]

Convert a vector graphics file into another format. This is done by
reading infile into Sketch and saving it as outfile. infile can be in
any format Sketch can read. The format of the output file is determined
by its extension.
"""

try:

    import cmdsyntax
    use_getopt = 0

except ImportError:

    import getopt
    use_getopt = 1

import string, sys, os

from Sketch import load, plugins
import Sketch

def convert(loader, infile, outfile, page_number):
    #doc = load.load_drawing(infile, **{"page": page_number})
    doc = loader.Load(page = page_number)
    extension = os.path.splitext(outfile)[1]
    fileformat = plugins.guess_export_plugin(extension)
    if fileformat:
        saver = plugins.find_export_plugin(fileformat)
        saver(doc, outfile)
    else:
        print "impconvert: unrecognized extension %s\n" % extension
        print "Try one of:"
        print
        for name, value in Sketch.Plugin.plugins.export_formats.items():
        
            print name, ":", string.join(value.extensions, ", ")
        
        print
        sys.exit(1)


def init_lib(argv):

    #Sketch.Base.config.load_user_preferences()
    Sketch.Issue(None, Sketch.const.INITIALIZE)
    Sketch.Graphics.init(argv, load_user_preferences = 1)
    Sketch.Plugin.init(argv, load_user_preferences = 1)

def find_plugin(format_name):

    plugins = Sketch.Plugin.plugins.import_plugins
    
    for plugin in plugins:
    
        if plugin.format_name == format_name:
        
            return plugin
    
    return None


def read_cmdsyntax_input(argv, syntax):

    syntax_obj = cmdsyntax.Syntax(syntax)
    
    matches, failed = syntax_obj.get_args(argv[1:], return_failed = 1)
    
    if matches == [] and cmdsyntax.use_GUI() != None:
    
        form = cmdsyntax.Form("Convert Impression document", syntax_obj, failed[0])
        
        matches = form.get_args()
    
    if len(matches) > 0:
    
        # Take the first match.
        match = matches[0]
    
    else:
    
        match = None
    
    return match


def read_getopt_input(argv):

    match = {}
    
    opt_dict = {"-s": "Starting page", "-f": "Finishing page"}
    arg_list = ["Input file", "Output directory", "File extension"]
    
    opts, args = getopt.getopt(argv[1:], "s:f:")
    
    # Read the options specified.
    
    for opt, value in opts:
    
        if opt_dict.has_key(opt):
         
            match[opt_dict[opt]] = value
        
        else:
        
            return None
    
    # Read the remaining arguments: there should be three of these.
    
    if len(args) != 3:
    
        return None
    
    else:
    
        for i in range(0, 3):
        
            match[arg_list[i]] = args[i]
    
    if match == {}: match = None
    
    return match


def main():
    init_lib(sys.argv)
    
    if use_getopt == 0:
    
        syntax = "<Input file> <Output directory> <File extension>" + \
            " [-s <Starting page>] [-f <Finishing page>]"
        
        match = read_cmdsyntax_input(sys.argv, syntax)
    
        if match is None:
        
            print "Usage: %s %s" % (sys.argv[0], syntax)
            print
            print "Convert an Impression file into another format. This is done by"
            print "reading the input file into Sketch and saving it as a series of"
            print "files in the output directory. The format of the output file is"
            print "determined by the extension specified."
            print
            print "Example: impconvert.py myfile.bc5 pages/ .ps -s 1 -f 3"
            print
            sys.exit()
    
    else:
    
        syntax = "[-s <Starting page>] [-f <Finishing page>] " + \
            "<Input file> <Output directory> <File extension>"
        
        match = read_getopt_input(sys.argv)
        
        if match is None:
        
            print "Usage: %s %s" % (sys.argv[0], syntax)
            print
            print "Convert an Impression file into another format. This is done by"
            print "reading the input file into Sketch and saving it as a series of"
            print "files in the output directory. The format of the output file is"
            print "determined by the extension specified."
            print
            print "Example: impconvert.py -s 1 -f 3 myfile.bc5 pages/ .ps"
            print
            sys.exit()
    
    # The input file
    infile = match["Input file"]
    
    # The output directory
    outdir = match["Output directory"]
    
    # The output file extension determines the file format to be used.
    file_ext = match["File extension"]
    
    try:
    
        if os.path.exists(outdir):
         
            if not os.path.isdir(outdir):
            
                raise OSError, "Directory not created"
        
        else:
        
            os.mkdir(outdir)
    
    except OSError:
    
        print "Could not create or write to output directory: %s" % outdir
        sys.exit(1)
    
    try:
    
        starting = int(match.get("Starting page", "1"))
        
        if starting < 1:
        
            raise ValueError, "Positive integers only"
    
    except ValueError:
    
        print "Please supply a positive integer for the page number."
        sys.exit(1)
    
    try:
    
        finishing = match.get("Finishing page", None)
        
        if finishing is not None:
         
            finishing = int(finishing)
            
            if finishing < starting:
            
                print "Please supply a finishing page number which equals or exceeds"
                print "the starting page number."
                sys.exit(1)
    
    except ValueError:
    
        print "Please supply a positive integer for the page number."
        sys.exit(1)
    
    # Find the relevant import plugin to use.
    plugin = find_plugin("Impression")
    
    if plugin is None:
    
        print "The Impression import plugin could not be found."
        sys.exit(1)
    
    # Instantiate the plugin as a loader.
    loader = plugin(None, infile, None)
    
    # Try to convert all the pages specified, aborting if a ValueError is
    # encountered.
    
    pages = len(loader.writer.document.chapter_pages)
    digits = len(str(pages))
    format = '%%0%ii' % digits
    
    # Examine each page.
    
    starting = min(max(1, starting), pages)
    
    if finishing is None:
    
        finishing = pages
    
    finishing = min(max(1, finishing, starting), pages)
    
    print
    print 'Writing pages:'
    print

    # Use a page counter to index the Drawfiles.
    
    for page_number in range(starting, finishing + 1):
    
        try:
        
            convert(
                loader, infile, os.path.join(
                    outdir, "page-" + (format % page_number) + file_ext
                    ),
                page_number
                )
        
        except ValueError:
        
            print
            print "Page number %i was not found in the document." % page_number
            sys.exit()
        
        sys.stdout.write("%i " % page_number)
        sys.stdout.flush()
    
    print

main()

